;**********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
;*        Solutions for real time microcontroller applications        *
;**********************************************************************
;*                                                                    *
;*           (C) 2006    SEGGER Microcontroller Systeme GmbH          *
;*                                                                    *
;*        Internet: www.segger.com    Support:  support@segger.com    *
;*                                                                    *
;**********************************************************************
;
;----------------------------------------------------------------------
;File        : init_AT91M40xxx_ROM.s
;Purpose     : startup file for use with external ROM/RAM
;---------------------------END-OF-HEADER------------------------------
;

;
; This module performs ROM/RAM remapping (if required), initializes stack 
; pointers and interrupts for each mode, and finally branches to __main in 
; the C library (which eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, 
; with IRQ and FIQ disabled.

        AREA    Init, CODE, READONLY
        PRESERVE8        
        
; --- ensure no functions that use semihosting SWIs are linked in from the C library
                IMPORT __use_no_semihosting_swi
; --- import OS_InitPLL to initialize the PLL
                IMPORT OS_InitPLL
       
; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled


        ENTRY

        EXPORT  Reset_Handler

Reset_Handler

; Remap by setting Remap bit of the CM_ctl register

        mov     r10, pc                 ; 
        add     r10, r10, #(EBI_init_table-boot_1)      ; points to init-Table
boot_1
        IMPORT |Image$$CODE$$Base|
        ; Load the address where to jump (in ROM)
        ldr     r11, =|Image$$CODE$$Base|   ; final target base address (depends on physical ROM address after remap !)
        mov     r12, pc                 ; To be position independent, calculate address offset
        add     r12, r12, #(after_remap-boot_2)    
boot_2                 
        mov     r12, r12, LSL #12       ; mask the top 12 bits
        mov     r12, r12, LSR #12       ; mask the top 12 bits
        add     r12, r12, r11           ; calculate final absolute address in ROM!!!

        
        ; Copy chip select register image to memory controller and command remap
        ldmia   r10!, {r0-r9,r11}       ; load the complete image and the EBI base
        stmia   r11!, {r0-r9}           ; store the complete image with the remap command

        ; jump to ROM at its new address
        ; this instruction was loaded into the pipeline before the remap was done
        mov     pc, r12                 ; jump and break the pipeline

        ; Put constant table here.
    LTORG
EBI_init_table 
  		  DCD   0x01002539                      ; CS0: Flash at 0x01000000, 16MB, 16 bits, very slow !
	      DCD   0x20000000                      ; CS1: unused
	      DCD   0x20000000                      ; CS2: unused
	      DCD   0x30000000                      ; CS3: unused
	      DCD   0x40000000                      ; CS4: unused
	      DCD   0x50000000                      ; CS5: unused              
	      DCD   0x60000000                      ; CS6: unused
	      DCD   0x70000000                      ; CS7: unused
	      DCD   0x00000001                      ; REMAP command
	      DCD   0x00000010                      ; early read
	      DCD   0xFFE00000                      ; EBI Base address

after_remap

; RAM is now at 0x0.
; The exception vectors (in vectors.s) must be copied from ROM to the RAM
; The copying is done later by the C library code inside __main


; --- Initialize stack pointer registers

; Enter each mode in turn and set up the stack pointer

; If FIQ, ABT and UND stack are required, a separate stack has to be initialized here

        IMPORT  ||Image$$CSTACK$$ZI$$Limit||       ; defined in scatter file
        IMPORT  ||Image$$IRQ_STACK$$ZI$$Limit||    ; defined in scatter file

        msr     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     sp, =||Image$$IRQ_STACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8

        msr     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     sp, =||Image$$CSTACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8

        ; ...
        bl OS_InitPLL
; --- Initialize critical IO devices

                
        IMPORT  __main

; --- Now enter the C code
        B       __main   ; note: use B not BL, because an application will never return this way


        END

