;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*       (C) 1997-2007    SEGGER Microcontroller Systeme GmbH        *
;*                                                                   *
;*       www.segger.com   Support:  support@segger.com               *
;*                                                                   *
;*********************************************************************
;
;----------------------------------------------------------------------
;File        : init_SoCLitePlus.s
;Purpose     : startup file for use with external RAM
;---------------------------END-OF-HEADER------------------------------
;

;
; This module performs ROM/RAM remapping (if required), initializes stack 
; pointers and interrupts for each mode, and finally branches to __main in 
; the C library (which eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, 
; with IRQ and FIQ disabled.


;*********************************************************************
;*
;*    Fixed values
;*
;*********************************************************************

; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

MODE_USR        EQU     0x10
MODE_FIQ        EQU     0x11
MODE_IRQ        EQU     0x12
MODE_SVC        EQU     0x13
MODE_ABT        EQU     0x17
MODE_UND        EQU     0x1B
MODE_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_BIT           EQU     0x80 ; when I bit is set, IRQ is disabled
F_BIT           EQU     0x40 ; when F bit is set, FIQ is disabled



;*********************************************************************
;*
;*    Reset handler
;*
;*********************************************************************

        PRESERVE8        
        AREA    Init, CODE, READONLY
        CODE32
        IMPORT  __program_start

; ************************************
;    SoCLite+ System Register files
; ************************************

INTC0               EQU     0x54006400
INTC1               EQU     0x54006404
INTC2               EQU     0x54006408
INTC3               EQU     0x5400640C
INTC4               EQU     0x54006410
INTC5               EQU     0x54006414
INTC6               EQU     0x54006418
INTC7               EQU     0x5400641C
INTSTA              EQU     0x54006420

TMC                 EQU     0x54006C08
WPC                 EQU     0x54004000
RMP                 EQU     0x54007400
APBW                EQU     0x54007800

MPMCWaitRead1       EQU     0x5400322C
MPMCStaticConfig2   EQU     0x54003240
MPMCStaticConfig3   EQU     0x54003260

;*********************************************************************
;       Reset Handler / Basic Initializations
;*********************************************************************

        ENTRY
        EXPORT  Reset_Handler

Reset_Handler
        ldr pc,=physical_addr  ; switch to physical program address
physical_addr                  ; to ensure correct addressing of constants after remapping

;*********************************************************************
;       Initialize SoCLite+ System Peripherals 
;*********************************************************************

; System Peripherals are not included in the SoCLite+ Testchip of the SoCLite+ Development Board.
; Therefore such an initialization shall be made at this point for SoCLite+ peripherals.

; ***  Memory Map & Multi-Port Memory Controller (MPMC) Configuration  ***

; The settings below for the MPMC have to be made by hardwiring !!


; ###### --> Configuration of Memory CS1 / FLASH area for Flash Program ##############

; --> Byte Lane State / MPMCSTCS1PB

; Set PB (byte line state) for CS1 - FLASH
; nMPMCBLSOUT(3:0) are all low during read access
; MPMCSTCS1PB = high level  -->  must be done hardwired

; --> Chip Select Polarity / MPMCSTCSxPOL

; Set POL (chip select polarity) for CS[3:0] - FLASH
; MPMCSTCS(3:0)POL are all low to provide active low chip select
; MPMCSTCS(3:0)POL = low level  -->  must be done hardwired

; --> Memory Bus Width / MPMCSTCS1MW[1:0]

; Set MW (memory width) for CS1 - FLASH
; MPMCSTCS1MW[1:0] are set to 10 to provide active low chip select
; MPMCSTCS1MW[1:0] = high/low level  -->  must be done hardwired

; ####### --> Configuration of Memory CS1 / FLASH area ##############

        mov     r0, #0x04           ; Speed up read access
        ldr     r1, =MPMCWaitRead1
        str     r0, [r1]

; ####### --> Configuration of Memory CS2 / SRAM area for Flash Download Program ##############

; Set:
; --> memory width to 32-bit
; --> CS2 low active
; --> byte lanes are all low for read/write

        mov     r0, #0x82
        ldr     r1, =MPMCStaticConfig2
        str     r0, [r1]

; ###### --> Configuration of Memory CS3 / SRAM area for Application Program (intermediate storage) ##############

; Set:
; --> memory width to 16-bit
; --> CS3 low active
; --> byte lanes are all low for read/write

        mov     r0, #0x81
        ldr     r1, =MPMCStaticConfig3
        str     r0, [r1]

; *** Remap Controller ***

        mov     r0, #0x1          ; internal RAM at address 0x0
        ldr     r1, =RMP          ; Remap bit is set
        str     r0, [r1]

; *** Write Protection ***

        mov     r0, #0x0          ; no write protection of internal RAM set
        ldr     r1, =WPC          ; is already the reset value
        str     r0, [r1]

; *** APB wait states ***

        mov     r0, #0x0          ; 0 waits on APB inserted
        ldr     r1, =APBW
        str     r0, [r1]

; *** Timer ***

        mov     r0, #0x0          ; timer disabled
        ldr     r1, =TMC          ; is already the reset value
        str     r0, [r1]

; *** Interrupts ***

        ldr     r0, =0x07070707     ; disable all interrupts, lowest priority
        ldr     r1, =INTC0          ; are already the reset values
        str     r0, [r1]
        ldr     r1, =INTC1
        str     r0, [r1]
        ldr     r1, =INTC2
        str     r0, [r1]
        ldr     r1, =INTC3
        str     r0, [r1]
        ldr     r1, =INTC4
        str     r0, [r1]
        ldr     r1, =INTC5
        str     r0, [r1]
        ldr     r1, =INTC6
        str     r0, [r1]
        ldr     r1, =INTC7
        str     r0, [r1]
        mov     r0, #0xFFFFFFFF     ; initially clear all interrupts
        ldr     r1, =INTSTA
        str     r0, [r1]

;*********************************************************************
;       Initialize Stack Pointers 
;*********************************************************************

; The stack pointers which are used for the OS are initialized.

; Add initialization needed before setup of stackpointers here

; We perform a SWI to ensure that we run in supervisor mode which enables mode switches
; This is necessary when program is executed from SoCLitePlus-SDRAM-loader, because
; CPU runs in USER mode then and mode switches are therefore disabled
; This switch only works if internal RAM is remapped before !

        mrs     r0, cpsr                            ; Test current mode
        and     r0, r0, #0x1f                       ;
        cmp     r0, #MODE_USR                       ; When we are in user-mode, we have to perform a mode switch
        bne     __setup_stacks                      ;
                
; When you arrive here during debug, please modify CPSR using the debugger.
; CPSR has to be set to SVC-Mode (0x13)
        
        ldr     r0, = __program_start+8             ; SWI handler address in startup
        ldr     r1, = 0x8                           ; SWI vector addres after remap
        ldr     r3, [r0]
        str     r3, [r1]                            ; initialize vector

        ldr     r0, = __setup_stacks                ; continue with "stack setup"
        ldr     r1, = 0x28                          ; SWI jump address
        str     r0, [r1]                            ; initialize vector
        swi     0x01                                ; perform mode switch

; Initialize the stack pointers.
; The pattern below can be used for any of the exception stacks:
; FIQ, IRQ, SVC, ABT, UND, SYS.
; The USR mode uses the same stack as SYS.
; The stack segments must be defined in the linker command file,
; and be declared above.

__setup_stacks
        IMPORT  ||Image$$CSTACK$$ZI$$Limit||       ; defined in scatter file
        IMPORT  ||Image$$IRQ_STACK$$ZI$$Limit||    ; defined in scatter file

        msr     CPSR_c, #MODE_IRQ:OR:I_BIT:OR:F_BIT ; No interrupts
        ldr     sp, =||Image$$IRQ_STACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8

        msr     CPSR_c, #MODE_SVC:OR:I_BIT:OR:F_BIT ; No interrupts
        ldr     sp, =||Image$$CSTACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8
         
; ***  --> From now on, the ARM CPU has to remain in Supervisor Mode !!!!!!!!

;*********************************************************************
;  Copy the exception vector table to internal ram
;*********************************************************************

        IMPORT  __vector_end
        IMPORT  ||Image$$MEM_EXEC$$Base||
        ldr     r0, =||Image$$MEM_EXEC$$Base||
        ldr     r1, =0x00000000
        ldr     r2, =__vector_end
        sub     r2, r2, r0
        mov     r2, r2, lsr #2
vector_copy_loop
        ldr     r3,[r0], #+4
        str     r3,[R1], #+4
        subs    r2,r2,#+0x1
        bne     vector_copy_loop

;*********************************************************************
;       Enter Main Precode 
;*********************************************************************


; --- Initialize critical IO devices and memory interface before segments and variables are initialized !
        IMPORT  __low_level_init
        ; Call  __low_level_init

        ldr     r0,=__low_level_init
        mov     lr, pc
        bx      r0

; The System has to enter main precode in SVC Mode !!!!!

        IMPORT  __main

; --- Now enter the C code
        b       __main   ; note: use B not BL, because an application will never return this way

        END


;*********************************************************************


